import 'dart:async';

import 'package:flutter/material.dart';
import 'package:kohana/kohana.dart';
import 'package:rxdart/rxdart.dart';

import 'base_widget_options.dart';

class BaseInputFields extends BaseViewModel {
  final BaseInputWidgetOptions options;
  final validationMessageSubject = BehaviorSubject<String?>();
  String? currentValidationMessage;
  final changesSubject = BehaviorSubject<String>();

  String? get validationMessage => validationMessageSubject.value;
  final TextEditingController controller;

  String get value => controller.text;

  Stream<String> get changes => changesSubject.stream;

  Stream<String?> asStream() => validationMessageSubject.stream.startWith(null);

  bool get showValidationMessage => options.validationStream == null;

  BaseInputFields._(this.options, this.controller);

  factory BaseInputFields(BaseInputWidgetOptions options) {
    var controller =
        options.controller ?? TextEditingController(text: options.value);
    return BaseInputFields._(options, controller);
  }

  void setValidationMessage(String? message, [bool show = true]) {
    currentValidationMessage = message;
    if (show || showValidationMessage) {
      validationMessageSubject.add(currentValidationMessage);
    }
  }

  @override
  FutureOr start() {
    var focusNode = options.focusNode;

    controller.addListener(() {
      changesSubject.add(controller.text);
    });

    focusNode?.addListener(() {
      if (options.onFocusRemoveError ??
          BaseInputWidgetOptions.defaultInstance.onFocusRemoveError == true) {
        if (focusNode.hasFocus) {
          setValidationMessage(null);
        }
      }
    });

    var inputValueStream = options.inputValueStream;

    if (inputValueStream != null) {
      disposeBag.addAll([
        inputValueStream.listen(
          (event) {
            if (event.value != value) {
              controller.text = event.value;
            }
            setValidationMessage(event.validationMessage, false);
          },
        )
      ]);
    }

    var listener = options.changeListener;
    if (listener != null) {
      disposeBag.addAll([
        changes.listen((event) {
          listener(InputValue(event,
              fromInput: true, validationMessage: validationMessage));
        }),
      ]);
    }

    var watcher = options.watcher;
    if (watcher != null) {
      disposeBag.addAll([
        watcher(this),
      ]);
    }

    var onMounted = options.onMounted;
    if (onMounted != null) {
      onMounted(this);
    }

    var validationStream = options.validationStream;
    if (validationStream != null) {
      disposeBag.addAll([
        validationStream
            .listen((_) => setValidationMessage(currentValidationMessage))
      ]);
    }
  }

  @override
  FutureOr dispose() {
    validationMessageSubject.close();
    changesSubject.close();
    controller.dispose();
    options.focusNode?.dispose();
    super.dispose();
  }
}
